library(abind)
x <- array(1:4, dim=c(2, 2), dimnames=list(X=c('a','b'), Y=c('d','e')))
y <- array(5:8, dim=c(2, 2), dimnames=list(W=c('A','B'), Z=c('D','E')))
# dimname names not used unless use.dnns=T
abind(x, y)
abind(x, y, along=1)
abind(x, y, use.dnns=T)
abind(y, x, use.dnns=T)
abind(y, x, use.dnns=T, use.first.dimnames=T)
abind(x, y, use.dnns=T, use.first.dimnames=T)
abind(x, y, along=1, use.dnns=T)
abind(y, x, along=1, use.dnns=T)
abind(y, x, along=1, use.dnns=T, use.first.dimnames=T)
abind(x, y, along=1, use.dnns=T, use.first.dimnames=T)
